"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hashArrayOfStrings = hashArrayOfStrings;
exports.hashString = hashString;
exports.hashStringSinglePass = hashStringSinglePass;
exports.hashSumOfIntArray = hashSumOfIntArray;
exports.md5Hex = md5Hex;
exports.revisionHash = revisionHash;

var _nodeCrypto = _interopRequireDefault(require("node:crypto"));

var _nodeBuffer = require("node:buffer");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * Create a hash for file revving. Creates an md5 hash from an input buffer or string,
 * and truncates it to 10 characters, which is unique enough for the purpose of file-revving.
 * @see https://blog.risingstack.com/automatic-cache-busting-for-your-css/
 * @param {Buffer | string} data
 * @returns {string} The truncated hash.
 */
function revisionHash(data) {
  if (typeof data !== "string" && !_nodeBuffer.Buffer.isBuffer(data)) throw new TypeError("Expected a Buffer or string");
  return _nodeCrypto["default"].createHash("md5").update(data).digest("hex").slice(0, 10);
}
/**
 * Computes a hash of an array of strings (the order of strings does not matter).
 * NOTE: An array with duplicate values given as parameter to this function
 * may yield to a hash which would collide with other hashes computed
 * on different arrays with this same function.
 * Therefore is on behalf of the caller to be sure that "array" contains
 * unique strings.
 * @param {string[]} array An array of strings.
 * @return {number} A number representing the hash code of the array.
 */


function hashArrayOfStrings(arr) {
  var code = 0;

  for (var i = 0; i < arr.length; i++) {
    var n = 0,
        curr = arr[i];

    for (var j = 0; j < curr.length; j++) {
      n = n * 251 ^ curr.charCodeAt(j);
    }

    code ^= n;
  }

  return code;
}
/**
 * Returns the hash of a string.
 * @see https://stackoverflow.com/questions/7616461/generate-a-hash-from-string-in-javascript#answer-7616484
 * @param {string} str The string.
 * @return {number} The hash code of the string, represented as a number.
 */


function hashString(str) {
  var hash = 0;
  if (str.length === 0) return hash;

  for (var i = 0; i < str.length; i++) {
    var chr = str.charCodeAt(i);
    hash = (hash << 5) - hash + chr;
    hash |= 0; // Convert to 32 bit integer.
  }

  return hash;
}
/**
 * A one-pass algorithm to compute the hash of a series of unique strings incrementally.
 * NOTE: Duplicate values given as parameter to this function may yield to a hash which would
 * collide with other hashes computed on different string series with this same function.
 * Therefore is on behalf of the caller to be sure that the series of strings will be unique
 * while calling this function incrementally.
 * @param {string} str The string.
 * @param {number} [startingHash] The previous hash computed with this same function if this
 * call is the continuation of the unique string series.
 * @return {number} The next hash.
 */


function hashStringSinglePass(str) {
  var startingHash = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var hash = startingHash;
  var n = 0;

  for (var j = 0; j < str.length; j++) {
    n = n * 251 ^ str.charCodeAt(j);
  }

  hash ^= n;
  return hash;
}
/**
 * Given a sequence of integers, computes the hash of their sum so that two different sequences
 * will have the same hash if the sum of their integers will be the same (the overall sum of the
 * sequences may be higher than `Number.MAX_SAFE_INTEGER`).
 * @see https://stackoverflow.com/questions/69749629/hash-function-that-returns-the-same-hash-for-a-sum-even-if-different-terms-lead/69749873
 * Example:
 * ```
 * hashSumOfIntArray([1, 2, 3, 4, 5, 6]); // 21
 * hashSumOfIntArray([1, 2, 3, 4, 5, 6]) === hashSumOfIntArray([2, 3, 1, 4, 6, 5]); // true
 * hashSumOfIntArray([1, 2, 3, 4, 5, 6]) === hashSumOfIntArray([5, 1, 10, 4, 1]); // true
 * hashSumOfIntArray([1, 2, 3, 4, 5, 6]) === hashSumOfIntArray([21]); // true
 * hashSumOfIntArray([1, 2, 3, 4, 5, 6]) === hashSumOfIntArray([7, 7, 7]); // true
 * ```
 * @param {number[]} seq A sequence of integers (an array).
 * @return {number} The hash of the sum of the sequence of integers.
 */


function hashSumOfIntArray(seq) {
  var BIG_PRIME = 4503599627370449;
  var hash = 0;

  for (var i = 0; i < seq.length; i++) {
    var value = seq[i];
    hash = hash % BIG_PRIME + value % BIG_PRIME;
  }

  return hash;
}
/**
 * Create a MD5 hash with hex encoding. Please don't use MD5 hashes for anything sensitive!
 * Works in the browser too, when used with a bundler like Webpack, Rollup, Browserify.
 * @param {Buffer | string | Array<Buffer | string>} data Prefer buffers as they're faster to hash, but strings can be useful for
 * small things. Pass an array instead of concatenating strings and/or buffers. The output
 * is the same, but arrays do not incur the overhead of concatenation.
 */


function md5Hex(data) {
  var hash = _nodeCrypto["default"].createHash("md5");

  var update = function update(buffer) {
    var inputEncoding = typeof buffer === "string" ? "utf8" : undefined;
    hash.update(buffer, inputEncoding);
  };

  if (Array.isArray(data)) {
    var _iterator = _createForOfIteratorHelper(data),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var element = _step.value;
        update(element);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  } else update(data);

  return hash.digest("hex");
}
//# sourceMappingURL=hash.js.map